# -*- coding: utf-8 -*-
import json
import time


def check_key_existed_from_cache(r_conn, prefix, staff_id):
    # redis_pwd = self.env["ir.config_parameter"].sudo()._api_get_parameter_redis_pwd()
    # r_obj = DwRedisPool(port=6379, db=0, password=redis_pwd)
    # r = r_obj.get_redis_connection()
    r = r_conn.get_redis_connection()
    if not r:
        return False
    else:
        user_key = prefix + str(staff_id)
        if r.exists(user_key):
            return True
        return False


def save_data_to_cache(r_conn, prefix, staff_id, dict_data, r_type='hash', save_remark=None, ffp=None):
    # r = self.connect_to_redis_srv()
    # redis_pwd = self.env["ir.config_parameter"].sudo()._api_get_parameter_redis_pwd()
    # r_obj = DwRedisPool(port=6379, db=0, password=redis_pwd)
    # r = r_obj.get_redis_connection()
    r = r_conn.get_redis_connection()
    # print('save_data_to_cache r: %s' % r)
    print(f'r@save_data_to_cache: {r}')
    if not r:
        return False
    else:
        user_key = prefix + str(staff_id)
        # zl_timestamp = str(int(time.time()))
        zl_timestamp = str(time.time())
        if save_remark:
            dict_data['save_remark'] = save_remark
        # dict_data_str = json.dumps(dict_data)
        dict_data_str = json.dumps(dict_data, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode('utf-8')
        # r_save_zl = r.hset(user_key, zl_timestamp, dict_data_str)
        # if r_save_zl:
        #     print(f'save success, timestamp: {zl_timestamp}')
        # else:
        #     print(f'save zl data to cache failed, timestamp: {prefix}_{staff_id}-{zl_timestamp}-{dict_data_str}')
        #     return False

        # del existed key if type diff
        if r.exists(user_key):
            current_type = r.type(user_key).decode('utf-8')
            # print(f'current_type: {current_type}')
            # print(f'r_type: {r_type}')
            # print(f"Current type of user_key '{user_key}': {current_type}")
            if current_type != r_type:
                r.delete(user_key)
                print(f"Deleted user_key '{user_key}' because its type was different.")

        retried_save = 3
        tried_time = 0
        while tried_time < retried_save:
            try:
                if r_type == 'hash':
                    r_save_zl = r.hset(user_key, zl_timestamp, dict_data_str)
                # elif r_type == 'string':
                else:
                    r_save_zl = r.set(user_key, dict_data_str)
                if r_save_zl:
                    # print(f'save success@cache: {user_key} - {zl_timestamp}')
                    # time.sleep(0.01)
                    return True
            except Exception as err:
                print(
                    f'err@save zl data to cache failed: {prefix}_{staff_id}-{zl_timestamp}-{dict_data_str}---{err}')
                tried_time += 1
                if tried_time > retried_save:
                    print(f"'retried save failed zl data: {json.dumps(dict_data, indent=4, sort_keys=True)}")
                    return False
                backoff = tried_time * 2
                print(f'Retrying in {backoff} seconds: count={tried_time}')
                time.sleep(backoff)


def del_field_line_from_cache(r, uk, f):
    retried_del = 3
    tried_time = 0
    while tried_time < retried_del:
        try:
            r_del = r.hdel(uk, f)
            if r_del:
                print(f'del field line from cache success, r_del: {r_del}')
                return True
        except Exception as err:
            print(
                f'del field line from cache failed, user key: {uk} and field: {f}---{err}')
            tried_time += 1
            if tried_time > retried_del:
                print(f"'retried del failed, user key: {uk} and field: {f}")
                return False
            backoff = tried_time * 2
            print(f'Retrying in {backoff} seconds: count={tried_time}')
            time.sleep(backoff)


# not del for line del, del pre for del key
def read_data_from_cache(r_conn, prefix, staff_id, not_del=None, del_pr=False):
    # r = self.connect_to_redis_srv()
    # redis_pwd = self.env["ir.config_parameter"].sudo()._api_get_parameter_redis_pwd()
    # r_obj = DwRedisPool(port=6379, db=0, password=redis_pwd)
    # r = r_obj.get_redis_connection()
    r = r_conn.get_redis_connection()
    if not r:
        return False
    else:
        user_key = prefix + str(staff_id)
        if r.exists(user_key):
            print(f"user key@read '{user_key}' existed")
            try:
                if prefix == 'dw_api_recovery_':
                    r_val = r.get(user_key)
                    return json.loads(r_val.decode('utf-8'))
                elif prefix in (
                        'dw_api_restart_',
                        'dw_api_pr_',
                        'dw_close_tbl_',
                        'dw_ws_tmnt_zl_',
                ):
                    return True
                else:
                    r_all_field = r.hkeys(user_key)
                    if len(r_all_field) == 1:
                        print('only one data')
                        r_field = float(r_all_field[0])
                    else:
                        print('more than one data')
                        sorted_list = sorted([float(x) for x in r_all_field])
                        r_field = sorted_list[0]
                    if r_field:
                        print(f'r_field: {r_field}')
                        r_val = r.hget(user_key, str(r_field))
                        print(f'r_val: {r_val}')
                        if not not_del:
                            # r_del = r.hdel(user_key, str(r_field))
                            del_field_line_from_cache(r, user_key, str(r_field))
                        return json.loads(r_val.decode('utf-8'))
            except Exception as err:
                print(f'read data from cache failed, user key: {user_key}---{err}')
                return False
            finally:
                if prefix in (
                        'dw_api_restart_',
                        # 'dw_api_pr_',
                        'dw_close_tbl_',
                        'dw_ws_tmnt_zl_',
                        # 'dw_ws_tmnt_poker_test_',
                        # 'dw_ws_tmnt_poker_verify_'
                ) and not not_del:
                    del_key_from_r(prefix, staff_id)
                if del_pr:
                    del_key_from_r(prefix, staff_id)
        else:
            # print(f"user key '{user_key}' not existed, no zl found!!!")
            return False


def del_key_from_r(r_conn, prefix, staff_id):
    # r = self.connect_to_redis_srv()
    # redis_pwd = self.env["ir.config_parameter"].sudo()._api_get_parameter_redis_pwd()
    # r_obj = DwRedisPool(port=6379, db=0, password=redis_pwd)
    # r = r_obj.get_redis_connection()
    r = r_conn.get_redis_connection()
    # r.ping()
    if not r:
        return False
    else:
        user_key = prefix + str(staff_id)
        print(f'user key@del: {user_key}')
        if r.exists(user_key):
            print(f"user key '{user_key}' existed")
            retried_del = 3
            tried_time = 0
            while tried_time < retried_del:
                r_del = r.delete(user_key)
                print(f'delete result: {r_del}')
                if r_del:
                    print(f'del success for user key: {user_key}')
                    # time.sleep(0.01)
                    return True
                else:
                    print(
                        f'del zl data from cache failed for {user_key}')
                    tried_time += 1
                    if tried_time > retried_del:
                        print(f"'retried del key failed for: {user_key}")
                        return False
                    backoff = tried_time * 2
                    print(f'Retrying in {backoff} seconds: count={tried_time}')
                    time.sleep(backoff)
            print(f'del finished!!!')
        else:
            return True



# -*- coding: utf-8 -*-
import json
import chardet


def check_charset(code_data):
    # import chardet
    charset = chardet.detect(code_data)['encoding']
    print('charset: %s' % charset)
    return charset


def format_dict_style(data_dict):
    try:
        return json.dumps(data_dict, indent=4, sort_keys=True, ensure_ascii=False, default=str)
        # return json.dumps(data_dict, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode('utf-8')
    except Exception as err:
        print('json.dumps error@itgeeker: %s' % err)

####################### deal list  #######################
# compare two list and return add and del list
def compare_lists_return_add_del(first_list, second_list):
    add_list = []
    del_list = []
    diff_list = list(
        set(first_list).symmetric_difference(
            set(second_list)))
    if diff_list:
        print('diff_list: %s' % diff_list)
        print('lists were different!!!')
        for diff in diff_list:
            if diff in first_list:
                del_list.append(diff)
            else:
                add_list.append(diff)
        print('add_list: %s' % add_list)
        print('del_list: %s' % del_list)
        return add_list, del_list
    else:
        return False, False


def convert_num_to_chinese_rmb(convert_number):
    """
    .转换数字为大写货币格式( format_word.__len__() - 3 + 2位小数 )
    convert_number 支持 float, int, long, string
    """
    format_word = ["分", "角", "元",
                   "拾", "百", "千", "万",
                   "拾", "百", "千", "亿",
                   "拾", "百", "千", "万",
                   "拾", "百", "千", "兆"]

    format_num = ["零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖"]
    if type(convert_number) == str:
        # - 如果是字符串,先尝试转换成float或int.
        if '.' in convert_number:
            try:
                convert_number = float(convert_number)
            except:
                raise Exception('%s   can\'t convert' % convert_number)
        else:
            try:
                convert_number = int(convert_number)
            except:
                raise Exception('%s   can\'t convert' % convert_number)

    if type(convert_number) == float:
        real_numbers = []
        for i in range(len(format_word) - 3, -3, -1):
            if convert_number >= 10 ** i or i < 1:
                real_numbers.append(
                    int(round(convert_number/(10**i), 2) % 10))

    elif isinstance(convert_number, (int, float)):
        real_numbers = [int(i) for i in str(convert_number) + '00']

    else:
        raise Exception('%s   can\'t convert' % convert_number)

    zflag = 0  # 标记连续0次数，以删除万字，或适时插入零字
    start = len(real_numbers) - 3
    convert_words = []
    for i in range(start, -3, -1):  # 使i对应实际位数，负数为角分
        if 0 != real_numbers[start - i] or len(convert_words) == 0:
            if zflag:
                convert_words.append(format_num[0])
                zflag = 0
            convert_words.append(format_num[real_numbers[start - i]])
            convert_words.append(format_word[i+2])

        elif 0 == i or (0 == i % 4 and zflag < 3):  # 控制 万/元
            convert_words.append(format_word[i+2])
            zflag = 0
        else:
            zflag += 1

    if convert_words[-1] not in (format_word[0], format_word[1]):
        # - 最后两位非"角,分"则补"整"
        convert_words.append("整")

    return ''.join(convert_words)


# -*- coding: utf-8 -*-

import time
import requests
import logging
_logger = logging.getLogger(__name__)


class GeekerApi:

    ####################### return data format #######################
    def format_return_data(self, isSuccess=True, errMsg='success', data=None, status=None, err_desc=None, isToast=None):
        data_returned = {
            'isSuccess': isSuccess,
            'errMsg': errMsg
        }
        if isToast:
            data_returned.update({
                'isToast': True
            })
        if data:
            data_returned.update({
                'data': data
            })
        if status:
            data_returned.update({
                'status': status
            })
        if err_desc:
            data_returned.update({
                'err_description': err_desc
            })
        # return json.dumps(error_returned_data)
        return data_returned

    ####################### dw post with tries #######################
    def send_post_with_retry(self, url, data, headers=None, max_retries=3, retry_interval=3):
        for i in range(max_retries):
            try:
                response = requests.post(url, data=data, headers=headers)
                response.raise_for_status()  # 抛出异常处理 HTTP 错误
                # return response
            except requests.exceptions.RequestException as err:
                _logger.warning(f"post data to srv failed: {err}")
                if i < max_retries - 1:
                    print(f"this is {i + 1} times failed request, try again in {retry_interval} seconds ...")
                    time.sleep(retry_interval)
                else:
                    _logger.warning(f"Retry {max_retries} times and fail, then give up")
                    raise  # 重新抛出异常，让调用者处理
                    # return False

    ####################### dw print httprequest info #######################
    def print_httprequest_info(self, req_quest_env, source):
        _logger.debug('/*-' * 8 + f'request httprequest source: {source}' +  '-*/' * 8)
        try:
            # _logger.info(f'request.session.sid@bus ws cntller: {request.session.sid}')
            # http_cookie = req_quest_env['HTTP_COOKIE']
            # _logger.info(f'from game srv http_cookie: {http_cookie}')
            # HTTP_COOKIE 'session_id=

            # http_authorization = req_quest_env['HTTP_AUTHORIZATION']
            # _logger.info(f'from game srv http_authorization: {http_authorization}')
            # http_authorization: Basic bzE2X2R3OjZjMDJjZDU0LWFiN2MtNDAyNy1iMjVlLTYyYzFlZWRlZjk1ZA ==

            http_user_agent = req_quest_env.get('HTTP_USER_AGENT', '')
            _logger.debug(f'http_user_agent: {http_user_agent}')
            # http_user_agent: PostmanRuntime/7.40.0

            # no these keys
            # client_ip_x = req_quest_env['X-Forwarded-For']
            # x_real_ip = req_quest_env['x-real-ip']
            # x_forwarded_host = req_quest_env['X-Forwarded-Host']

            remote_addr = req_quest_env.get('REMOTE_ADDR', '')
            _logger.info(f'remote_addr: {remote_addr}')

            http_host = req_quest_env.get('HTTP_HOST', '')
            _logger.debug(f'http_host: {http_host}')
        except Exception as err:
            _logger.error('err@logger request env params: %s' % err)



# -*- coding: utf-8 -*-
import re

####################### cv parse #######################
# extract company pattern_none_greedy
def filter_co_name(co_name, ban_company_list):
    if ban_company_list:
        if co_name not in ban_company_list:
            if co_name.endswith('集团'):
                if 3 < len(co_name) < 10:
                    return co_name
            elif 5 < len(co_name) < 18:
                return co_name
            else:
                return False
    else:
        if co_name.endswith('集团'):
            if 3 < len(co_name) < 10:
                return co_name
        elif 5 < len(co_name) < 18:
            return co_name
        else:
            return False

def parse_company_name(text_str, ban_company_list):
    pattern_none_greedy = re.compile(r'(\w+?)(公司|集团)(\s)')
    if text_str:
        if isinstance(text_str, bytes):
            text_str = text_str.decode('utf-8')
            print('convert bytes text_str to string @geekermaster_api_parse')
        extract_res_list = re.findall(pattern_none_greedy, text_str)
        print('extract_res_list: %s' % extract_res_list)
        if extract_res_list:
            found_company_list = []
            for erl in extract_res_list:
                found_company_list.append("".join(erl))
            print('found_company_list: %s' % found_company_list)
            if found_company_list:
                final_list = []
                # ban_company_list = generate_ban_company_list()
                for fcl in found_company_list:
                    fcl = fcl.strip()
                    filtered_name = filter_co_name(fcl, ban_company_list)
                    if filtered_name:
                        final_list.append(filtered_name)
                    # if fcl.endswith('集团'):
                    #     if 3 < len(fcl) < 10:
                    #         final_list.append(fcl)
                    # elif 5 < len(fcl) < 18:
                    #     if ban_company_list:
                    #         if fcl not in ban_company_list:
                    #             final_list.append(fcl)
                    #     else:
                    #         final_list.append(fcl)
                print("will return final_list: %s", final_list)
                if final_list:
                    return final_list[0]
    return False

def parse_school_name(text_str):
    pattern_none_greedy = re.compile(r'(\w+?)(大学)')
    if text_str:
        if isinstance(text_str, bytes):
            text_str = text_str.decode('utf-8')
            print('convert text_str to string @geekermaster_api_parse')
        extract_res_list = re.findall(pattern_none_greedy, text_str)
        print('extract_res_list: %s' % extract_res_list)
        if extract_res_list:
            found_school_list = []
            for erl in extract_res_list:
                found_school_list.append("".join(erl))
            print('found_school_list: %s' % found_school_list)
            if found_school_list:
                final_list = []
                # ban_school_list = generate_ban_school_list()
                for fcl in found_school_list:
                    # if fcl.strip() not in ['有限公司', '目前公司', '所在公司', '给贵公司']:
                    if 3 < len(fcl.strip()) < 12:
                        # if ban_school_list:
                        #     if fcl.strip() not in ban_school_list:
                        #         final_list.append(fcl)
                        # else:
                        final_list.append(fcl)
                print("will return final_list: %s", final_list)
                if final_list:
                    return final_list[0]
    return False

def parse_gender_name(text_str):
    pattern_none_greedy = re.compile(r'(\s)(男|女|male|female)(\s)', re.I)
    if text_str:
        if isinstance(text_str, bytes):
            text_str = text_str.decode('utf-8')
            print('parse text_str for gender')
        extract_res_list = re.findall(pattern_none_greedy, text_str.lower())
        print('extract_res_list: %s' % extract_res_list)
        if extract_res_list:
            found_gender_list = []
            for erl in extract_res_list:
                found_gender_list.append("".join(erl))
            print('found_gender_list: %s' % found_gender_list)
            if found_gender_list:
                final_list = []
                # ban_gender_list = generate_ban_gender_list()
                for fcl in found_gender_list:
                    final_list.append(fcl.strip())
                print("will return final_list: %s", final_list)
                if final_list[0] in ['男', 'Male', 'male']:
                    return '男'
                elif final_list[0] in ['女', 'Female', 'female']:
                    return '女'
    return False

def check_chinese_id_number(num_str):
    str_to_int = {'0': 0, '1': 1, '2': 2, '3': 3, '4': 4, '5': 5,
                  '6': 6, '7': 7, '8': 8, '9': 9, 'X': 10}
    check_dict = {0: '1', 1: '0', 2: 'X', 3: '9', 4: '8', 5: '7',
                  6: '6', 7: '5', 8: '4', 9: '3', 10: '2'}
    if len(num_str) != 18:
        raise Exception('请输入标准的第二代身份证号码')
    check_num = 0
    for index, num in enumerate(num_str):
        if index == 17:
            right_code = check_dict.get(check_num % 11)
            if num == right_code:
                # print(u"身份证号: %s 校验通过" % num_str)
                return True
            else:
                #         print(u"身份证号: %s 校验不通过, 正确尾号应该为：%s" % (num_str, right_code))
                # check_num += str_to_int.get(num) * (2 ** (17 - index) % 11)
                raise Exception('身份证号: %s 校验不通过, 正确尾号应该为：%s' % (num_str, right_code))
        check_num += str_to_int.get(num) * (2 ** (17 - index) % 11)


# -*- coding: utf-8 -*-
import json
import os
import tempfile
import time
from datetime import datetime, timezone


####################### operate dict to json  file #######################
def save_data_to_ffp(prefix, staff_id, data, save_remark=None):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'save zl to target_ffp: {target_ffp}')
    if not os.path.isfile(target_ffp):
        try:
            save_dict = {
                'list': [data],
            }
            utc_now = datetime.now(timezone.utc)
            save_dict['save_utc_timestamp'] = utc_now.timestamp()
            if save_remark:
                save_dict['save_remark'] = save_remark
            print(f'save_dict final: {json.dumps(save_dict, indent=4, sort_keys=True)}')
            retried_save = 8
            tried_time = 0
            while tried_time < retried_save:
                try:
                    with open(target_ffp, 'wb') as jf_new:
                        jf_new.write(
                            json.dumps(save_dict, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode(
                                'utf-8'))
                    return True
                except (IOError, OSError) as err:
                    print(f"file is occupied @save zl to ffp:{err}")
                except Exception as err:
                    print('err@save zl to ffp: %s' % err)
                time.sleep(0.5)
                tried_time += 1
                if tried_time > retried_save:
                    print(f"'retried save zl to ffp failed for: ID-{staff_id}")
                    return False
        except Exception as err:
            print('err@save zl to json ffp: %s' % err)
            return False
    else:
        update_data_to_ffp(prefix, staff_id, data, update_remark='update existed zl ffp')


def update_data_to_ffp(prefix, staff_id, data_new, update_remark=None):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'update to target_ffp: {target_ffp}')
    try:
        with open(target_ffp, 'rb') as jf_old:
            updated_dict = json.loads(jf_old.read())
        # for k, v in dict_up_data.items():
        #     updated_dict[k] = v
        # data_list = updated_dict['data_list']
        data_list = updated_dict.get('data_list', [])

        # for none dedup
        if data_new not in data_list:
            data_list.append(data_new)

        # # for dedup
        # data_newnone_read_list = []
        # for zl in data_list:
        #     if 'is_read' in zl:
        #         data_newrm_read = {k: v for k, v in zl.items() if k != 'is_read'}
        #         data_newnone_read_list.append(data_newrm_read)
        #     else:
        #         data_newnone_read_list.append(zl)
        # print(f'data_newnone_read_list: {data_newnone_read_list}')
        # if data_newnone_read_list:
        #     if data_new not in data_newnone_read_list:
        #         data_list.append(data_new)
        #     else:
        #         _logger.info('/*-' * 6 + ' data_new already in data_list ' + '-*' * 6)
        # else:
        #     data_list.append(data_new)

        utc_now = datetime.now(timezone.utc)
        updated_dict['update_utc_timestamp'] = utc_now.timestamp()
        if update_remark:
            updated_dict['update_remark'] = update_remark
        print(f'updated zl dict final: {json.dumps(updated_dict, indent=4, sort_keys=True)}')
        retried_up = 8
        tried_time = 0
        while tried_time < retried_up:
            try:
                with open(target_ffp, 'wb') as jf_new:
                    jf_new.write(
                        json.dumps(updated_dict, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode(
                            'utf-8'))
                return True
            except (IOError, OSError) as err:
                print(f"file is occupied @update zl to ffp:{err}")
            except Exception as err:
                print('err@update zl to ffp: %s' % err)
            time.sleep(0.5)
            tried_time += 1
            if tried_time > retried_up:
                print(f"'retried update zl to ffp failed for: ID-{staff_id}")
                return False
    except Exception as err:
        print('err@update dict ffp zl json ffp: %s' % err)
        return False


def _get_and_delete_first_cmd(data_dict, cmd_key, not_del=None):
    if cmd_key not in data_dict or not isinstance(data_dict[cmd_key], list) or not data_dict[cmd_key]:
        return False

    # for none dedup
    else:
        first_cmd = data_dict[cmd_key][0]
        print("First cmd:", first_cmd)
        if not not_del:
            data_dict[cmd_key].pop(0)
            print(f'data_dict: {data_dict}')
        return first_cmd

    # # for dedup
    # cmd_list = [x for x in data_dict[cmd_key] if 'is_read' not in x]
    # print(f'cmd_list: {cmd_list}')
    # if not cmd_list:
    #     return None
    # else:
    #     first_cmd = cmd_list[0]
    #     # first_cmd = data_dict[cmd_key][0]
    #     print("First cmd:", first_cmd)
    #     if not not_del:
    #         # data_dict[cmd_key].pop(0)
    #         first_cmd['is_read'] = True
    #         print(f'data_dict: {data_dict}')
    #     return first_cmd


def read_data_to_ffp(prefix, staff_id, key_n, not_del=None):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'read zl from target_ffp: {target_ffp}')
    if not os.path.isfile(target_ffp):
        return False
    try:
        with open(target_ffp, 'rb') as jf_old:
            read_dict = json.loads(jf_old.read())
        # print(f'read_dict: {read_dict}')
        first_cmd = _get_and_delete_first_cmd(read_dict, key_n, not_del)
        print(f'read_dict after read and rm: {json.dumps(read_dict, indent=4, sort_keys=True)}')
        retried_read = 8
        tried_time = 0
        while tried_time < retried_read:
            try:
                with open(target_ffp, 'wb') as jf_new:
                    jf_new.write(
                        json.dumps(read_dict, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode(
                            'utf-8'))
                    return first_cmd
            except (IOError, OSError) as err:
                print(f"file is occupied @read zl to ffp:{err}")
            except Exception as err:
                print('err@read zl to ffp: %s' % err)
            time.sleep(0.5)
            tried_time += 1
            if tried_time > retried_read:
                print(f"'retried read zl to ffp failed for: ID-{staff_id}")
                return False
    except Exception as err:
        print('err@read dict ffp zl json ffp: %s' % err)
        return False


####################### deal with file write and open read #######################
# tmp_ffp = os.path.join(tmp_path, 'dw_dealer_' + str(dealer_id) + 'latest_verify')

def try_os_remove_ffp(file_ffp):
    retried_del = 8
    tried_time = 0
    while tried_time < retried_del:
        try:
            os.remove(file_ffp)
            return True
        except PermissionError as err:
            print(f'err@try os remove ffp PermissionError-(indicating the file is occupied: {err}')
            time.sleep(0.5)
            tried_time += 1
            if tried_time > retried_del:
                print(f"'retried os remove ffp failed for ffp: {file_ffp}")
                return False
        except Exception as err:
            print('err@try os remove ffp(%s) failed: %s' % (file_ffp, err))
            return False


def check_ffp_existed(prefix, staff_id, validity_seconds=2):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'check ffp existed: {target_ffp}')
    if os.path.isfile(target_ffp):
        with open(target_ffp, 'rb') as jf:
            res_value = json.loads(jf.read())
            create_ts = res_value.get('cur_utc_timestamp', None)
            create_dt = datetime.fromtimestamp(create_ts)
            # now_ts = (datetime.now(timezone.utc)).timestamp()
            # now_dt = datetime.fromtimestamp(int_timestamp)
            now_dt = datetime.now()
            ts_diff = (now_dt - create_dt).total_seconds()
            print('ts_diff: %s' % ts_diff)
            if ts_diff > validity_seconds:
                return False
        return True
    return False


def save_dict_data_to_ffp(prefix, staff_id, dict_data, save_remark=None):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'save to target_ffp: {target_ffp}')
    utc_now = datetime.now(timezone.utc)
    dict_data.update({
        'cur_utc_timestamp': utc_now.timestamp()
    })
    if save_remark:
        dict_data.update({
            'save_remark': save_remark
        })
    # with open(target_ffp, 'wb') as jf:
    #     jf.write(json.dumps(dict_data, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode('utf-8'))
    retried_save = 6
    tried_time = 0
    while tried_time < retried_save:
        try:
            with open(target_ffp, 'wb') as jf:
                jf.write(
                    json.dumps(dict_data, indent=4, sort_keys=True, ensure_ascii=False, default=str).encode('utf-8'))
            return True
        except (IOError, OSError) as err:
            print(f"file is occupied @save dict to ffp:{err}")
            time.sleep(0.5)
            tried_time += 1
            if tried_time > retried_save:
                print(f"'retried save dict to ffp failed for: ID-{staff_id}")
                return False
        except Exception as err:
            print('err@write dict to ffp: %s' % err)
            return False
    return True


def read_dict_value_from_ffp(prefix, staff_id, last_stmp=None, dict_key=None, del_pr=False, not_del=False):
    tmp_path = tempfile.gettempdir()
    source_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    # print(f'read from source_ffp: {source_ffp}')
    try:
        if not os.path.isfile(source_ffp):
            return False
        elif prefix in (
                'dw_api_restart_',
                'dw_api_pr_',
                'dw_close_tbl_',
                'dw_ws_tmnt_data_new',
                'dw_ws_tmnt_poker_test_',
                'dw_ws_tmnt_poker_verify_'
        ):
            return True
        else:
            with open(source_ffp, 'rb') as jf:
                res_value = json.loads(jf.read())
                if dict_key:
                    res_value = res_value.get(dict_key, None)
                    return res_value
            print(f'res_value: {res_value}')
            # if last_stmp <= res_value.get('cur_utc_timestamp', None):
            #     return res_value
            # return False
            return res_value
    except Exception as err:
        print(f'err@read source_ffp failed: %s' % err)
        return False
    finally:
        if os.path.isfile(source_ffp) and prefix != 'dw_api_pr_' and not not_del:
            # os.remove(source_ffp)
            try_os_remove_ffp(source_ffp)
        if del_pr:
            # os.remove(source_ffp)
            try_os_remove_ffp(source_ffp)


def del_ffp(prefix, staff_id):
    tmp_path = tempfile.gettempdir()
    target_ffp = os.path.join(tmp_path, prefix + str(staff_id) + '.json')
    print(f'del zl ffp: {target_ffp}')
    if os.path.isfile(target_ffp):
        retried_del = 8
        tried_time = 0
        while tried_time < retried_del:
            try:
                os.remove(target_ffp)
                return True
            except PermissionError as err:
                print(f'err@del zl ffp PermissionError-(indicating the file is occupied: {err}')
                time.sleep(0.5)
                tried_time += 1
                if tried_time > retried_del:
                    print(f"'retried del ffp failed for: ID-{staff_id}")
                    return False
            except Exception as err:
                print('err@del ffp(%s) failed: %s' % (target_ffp, err))
                return False
    else:
        return True


from datetime import datetime, date

from itgeeker.date_geeker import is_summer_vacation

if __name__ == '__main__':
    today = date.today()
    if is_summer_vacation(today):
        print(f"{today} is during summer vacation.")
    else:
        print(f"{today} is not during summer vacation.")

    # Test with a specific date
    test_date = date(2023, 8, 15)
    if is_summer_vacation(test_date):
        print(f"{test_date} is during summer vacation.")
    else:
        print(f"{test_date} is not during summer vacation.")


 #python setup.py bdist_wheel

from setuptools import setup, find_packages

setup(
    name="itgeeker",
    version="0.1.0",
    packages=find_packages(exclude=["main.py"]),
    # 你可以根据需要添加更多配置项
    author="ITGeeker",
    description="ITGeeker Tools Package",
    python_requires=">=3.6",
)

from setuptools import setup, find_packages

setup(
    name='your_package_name',  # 替换为你的包名
    version='1.0.0',  # 替换为你的版本号
    packages=find_packages(),  # 自动查找所有包
    include_package_data=True,  # 包含额外的数据文件（如README.md，LICENSE等）
    install_requires=[  # 声明依赖的包
        'requests',
        'beautifulsoup4'
    ],
    entry_points={  # 可选：定义命令行入口
        'console_scripts': [
            'your_command = your_package.module:your_function'
        ]
    },
    #  可选：其他元数据，如作者、描述、许可证等
)


from itgeeker.geekermaster_parse import parse_company_name, parse_school_name, parse_gender_name


ban_company_list = ['上海奇客罗方智能科技有限公司', '奇客罗方智能科技']
print(parse_company_name('来自 上海奇客罗方智能科技有限公司 的计技术奇客', ban_company_list))


from itgeeker import geekermaster_parse
ban_company_list = ['奇客罗方智能科技']
print(geekermaster_parse.parse_company_name('来自 上海奇客罗方智能科技有限公司 的计技术奇客', ban_company_list))


# code to test parse_school_name
print(parse_school_name('上海交通大学'))

print('\n' + '/*--*/' * 18)
# code to test parse_gender_name
print(parse_gender_name(' 男 '))
print(parse_gender_name(' 男生'))



# -*- coding: utf-8 -*-
import hashlib
from urllib import parse

####################### md5 sign #######################
def md5_sign(attributes, mct_key):
    # _logger.warning(f'attributes for md5 sign: {attributes}')
    attributes_new = {k: attributes[k] for k in sorted(attributes.keys()) if
                      attributes[k] is not None and k != 'stream_url'}
    # print(f'attributes_new@md5 sign: {attributes_new}')
    # sign_str = "&".join(
    #     [f"{sk}={attributes_new[sk]}" for sk in attributes_new.keys()]
    # )
    sign_str = parse.urlencode(attributes_new)
    # print(f'sign_str@md5 sign: {sign_str}')
    return (
        hashlib.md5((sign_str + "&key=" + mct_key).encode(encoding="utf-8"))
        .hexdigest()
        .upper()
    )


####################### sha-256 sign #######################
def sha256_sign(attributes, api_key):
    """ 生成 SHA-256 签名 """
    # 1. 过滤非空参数并排序
    attributes_new = {
        k: attributes[k]
        for k in sorted(attributes.keys())
        if attributes[k] is not None
    }

    # 2. 生成待签名字符串（保持 urlencoded 编码规范）
    # sign_str = parse.urlencode(attributes_new)
    # 生成待签名字符串（保持空格不被编码）
    def custom_quote(value):
        return parse.quote(value, safe="").replace("%20", " ")

    sign_str = "&".join(
        f"{custom_quote(str(k))}={custom_quote(str(v))}" for k, v in attributes_new.items()
    )
    print(f"sign_str: {sign_str}")

    # 3. 拼接密钥并生成 SHA-256 签名
    raw_sign = f"{sign_str}&key={api_key}".encode("utf-8")
    print(f"raw_sign: {raw_sign}")
    sign_code = hashlib.sha256(raw_sign).hexdigest().upper()
    print(f"sign_code: {sign_code}")
    return sign_code



# -*- coding: utf-8 -*-
from sys import platform


def return_platform():
    if platform == "linux" or platform == "linux2":
        return 'linux'
    elif platform == "win32":
        return 'windows'
    else:
        return 'other platform'



# -*- coding: utf-8 -*-
import os
from sys import platform


def generate_files_root_path(folder_usage, cur_db=None, o_main_ver=None):
    path_files_root = ''
    # cur_db = self.env.cr.dbname
    # o_main_ver = self.get_odoo_latest_ver()
    if platform == "linux" or platform == "linux2":
        # print('linux')
        path_files_root = os.path.join('/opt/odoo' + str(o_main_ver) + '/db_' + cur_db, folder_usage)
    elif platform == "win32":
        # print('# Windows...')
        path_files_root = os.path.join(r'D:\opt\odoo' + str(o_main_ver) + '\\db_' + cur_db, folder_usage)
    elif platform == "darwin":
        print('OS X, not setup now')
        path_files_root = os.path.join('/opt/odoo' + str(o_main_ver) + '/db_' + cur_db, folder_usage)
    if not os.path.isdir(path_files_root):
        os.makedirs(path_files_root)
    print('path_files_root@common: %s' % path_files_root)
    return path_files_root


####################### deal with agrs domain #######################
def replace_square_brackets(my_list):
    my_list_str = str(my_list)
    new_str = my_list_str.replace("[", "(").replace("]", ")")
    if new_str.startswith('(((') and new_str.endswith(')),)'):
        new_str = new_str.replace('(((', '[(').replace(')),)', ')]')
    print(f'new_str: {new_str}')
    new_list = eval(new_str)
    print(f'new_list type: {type(new_list)}')
    return new_list

####################### format data for postgresql sql #######################
def convert_dict_psql_up_date_arr(dict_data):
    arr = []
    for key, value in dict_data.items():
        # if type(value) is int:  # Handle Integers
        if type(value) in (int, float):  # Handle digits
            arr.append("{key} = {value}".format(key=key, value=value))
        else:  # Default Handler
            arr.append("{key} = '{value}'".format(key=key, value=value))
    # generate string from key/pair array
    data_arr = ", ".join(arr)
    print(f'data_arr: {data_arr}')
    return data_arr

def convert_dict_psql_insert_into_data_arr(dict_data):
    columns = dict_data.keys()
    values = [dict_data[column] for column in columns]
    # values = []
    # for column in columns:
    #     if dict_data[column] == 'NULL':
    #         values.append(None)
    #     else:
    #         values.append(dict_data[column])
    print(f"psql columns: {','.join(columns)}")
    print(f"psql values: {tuple(values)}")
    return ','.join(columns), tuple(values)


# print logger args
def _logger_print_args(fun_name, args, kwargs, staff_id=None):
    # _logger.warning(f'/*--*/ from function: {fun_name} for ID-{str(staff_id)} /*--*/')
    print('/*--*/ from function: %s for ID-%s /*--*/', fun_name,
                  staff_id if staff_id is not None else 'Unknown')
    if args:
        print(f'args: {args}')
    if kwargs:
        print(f'kwargs: {kwargs}')

####################### deal with photo #######################
def get_avatar_data_uri(model_name, obj_id, img_f_name='image_1920'):
    target_obj = self.env[model_name].browse(obj_id)
    if not target_obj:
        _logger.warning("Target object with id %s does not exist.", target_obj.id)
        img_data = None

    # 检查字段是否存在
    if hasattr(target_obj, '_fields') and img_f_name not in target_obj._fields:
        _logger.warning("Field %s does not exist in model %s.", img_f_name, model_name)
        return None

    img_data = getattr(target_obj, img_f_name, None)

    if not img_data:
        _logger.info("No image data found for field %s in object %s.", img_f_name, obj_id)
        return None

    raw = base64.b64decode(img_data)
    print(f'raw@get_avatar_data_uri: {raw}')

    # 判断 MIME 类型
    if raw.startswith(b'\x89PNG'):
        mime = "image/png"
    elif raw.startswith(b'\xff\xd8'):
        mime = "image/jpeg"
    elif raw.strip().startswith(b'<svg') or raw.strip().startswith(b'<?xml version='):
        mime = "image/svg+xml"
    else:
        return None
        # mime = "application/octet-stream"

    return f"data:{mime};base64,{img_data.decode()}"

####################### deal with user #######################
def get_cur_user_id(self):
    return self.env.user.id

# ####################### new env #######################
# def fetch_sql_result(query, params, db_name, fetch_one=False):
#     # db_name = self.env.cr.dbname
#     with registry(db_name).cursor() as new_cr:
#         try:
#             new_cr.execute(query, params)
#             if fetch_one:
#                 return new_cr.fetchone()
#             return new_cr.fetchall()
#         except Exception as e:
#             _logger.error(f"SQL 查询出错：{e}")
#             return None


# -*- coding: utf-8 -*-
import calendar
from datetime import datetime, date, timedelta
import pytz

tday = date.today()
DEFAULT_DATE_FORMAT = '%Y-%m-%d'
DEFAULT_DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'

def get_start_time(yr, mo, d):
    start_time = datetime(int(yr), int(mo), int(d), 0, 0, 0)
    print(f'start_time: {start_time}')
    return start_time


def get_end_time(yr, mo, d=None):
    if not d:
        d = calendar.monthrange(yr, mo)[1]
        print(f"The last day of the month is: {d}")
    end_time = datetime(int(yr), int(mo), int(d), 23, 59, 59)
    print(f'end_time: {end_time}')
    return end_time


def get_pre_yr_mo(yr, mo):
    d = date(yr, mo, 1)
    last_month = d - timedelta(days=1)
    print(f'pre mo yr: {last_month.year}, pre mo mo: {last_month.month}')
    return last_month.year, last_month.month


def get_last_day_of_month(year, month):
    """
  This function takes a year and month as input and returns the last day of the month.
  """
    _, num_days = calendar.monthrange(year, month)
    print(f'last day of the month: {num_days}')
    return num_days

####################### deal with time #######################
# convert second to x hours x minutes x seconds
def format_seconds_duration(seconds):
    try:
        duration = timedelta(seconds=seconds)
        hours, remainder = divmod(duration.total_seconds(), 3600)
        minutes, seconds = divmod(remainder, 60)
        # return f"{int(hours)} hours {int(minutes)} minutes {int(seconds)} seconds"
        # return f"{int(hours)}小时{int(minutes)}分{int(seconds)}秒"
        if minutes > 0 and seconds > 0:
            return f"{int(hours)}小时{int(minutes)}分{int(seconds)}秒"
        elif minutes > 0:
            return f"{int(hours)}小时{int(minutes)}分"
        elif seconds > 0:
            return f"{int(hours)}小时{int(seconds)}秒"
        else:
            return f"{int(hours)}小时"
    except Exception as err:
        print('err@format seconds duration: %s' % err)
        return ''

# 该方法返回自 Unix 纪元以来的毫秒数，并始终使用 UTC 表示时间js const utcTimestamp = new Date().getTime();
def convert_timestamp_to_dt(cur_utc_timestamp):
    try:
        print('not million sec timestamp')
        last_time = datetime.fromtimestamp(int(cur_utc_timestamp))
    except (ValueError, OSError):
        print('it is million sec timestamp / 1000')
        last_time = datetime.fromtimestamp(int(cur_utc_timestamp) / 1000)
    return last_time

def get_time_with_tz(zone_name='Asia/Shanghai'):
    # Get the Beijing timezone - 'Asia/Shanghai'
    location_tz = pytz.timezone(zone_name)
    utc_now = datetime.now(pytz.utc)
    cur_time_with_tz = utc_now.astimezone(location_tz)
    # print(f'cur_time_with_tz in bj tz: {cur_time_with_tz}')
    return cur_time_with_tz

def remove_time_zone(dt_with_tz):
    dt_no_tz = dt_with_tz.replace(tzinfo=None)
    print("removed datetime tz:", dt_no_tz)
    return dt_no_tz

def convert_datetime_to_utc(dt_with_tz):
    dt_utc_with_tz = dt_with_tz.astimezone(pytz.utc)
    print("convert to UTC with tz:", dt_utc_with_tz)
    return dt_utc_with_tz

def convert_dt_with_tz(dt, tz_name='Asia/Shanghai'):
    # set as tz
    bj = pytz.timezone('Asia/Shanghai')
    dt_bj_tz = bj.localize(dt)
    print("dt_bj_tz:", dt_bj_tz)

    # convert to new tz
    new_tz = pytz.timezone(tz_name)
    dt_to_spec_tz = dt_bj_tz.astimezone(new_tz)
    print("dt_to_spec_tz:", dt_to_spec_tz)
    return dt_to_spec_tz

def check_two_time_diff_sec(cur_time, last_time):
    print(f'cur_time: {cur_time}')
    print(f'last_time: {last_time}')
    try:
        diff_sec = (cur_time - last_time).total_seconds()
        print(f'diff_sec: {diff_sec}')
        return diff_sec
    except Exception as err:
        print('err@check two time diff: %s' % err)
        return 0

def is_summer_vacation(cur_date=None):
    if cur_date is None:
        cur_date = date.today()
    year = cur_date.year
    start = date(year, 7, 1)
    end = date(year, 9, 30)
    return start <= cur_date <= end


